#include "stdafx.h"
#include "cphidgetled.h"
#include "cusb.h"
#include "csocket.h"
#include "cthread.h"

static int CCONV _sendpacket(CPhidgetLEDHandle phid,
    unsigned int index, unsigned int power)
{
again:
	if (!CPhidget_isAttached(&phid->phid))
	{
		return EPHIDGET_NOTATTACHED;
	}
	CThread_mutex_lock(&phid->phid.outputLock);
	if (phid->changedLED_Power[index]) {
		if (phid->nextLED_Power[index] != power) {
			CThread_mutex_unlock(&phid->phid.outputLock);
			CThread_wait_on_event(&phid->phid.writtenEvent, INFINITE);
			goto again;
		} else {
			CThread_mutex_unlock(&phid->phid.outputLock);
			return EPHIDGET_OK;
		}
	} else {
		if (phid->LED_Power[index] == power) {
			CThread_mutex_unlock(&phid->phid.outputLock);
			return EPHIDGET_OK;
		}
		phid->changeRequests++;
		phid->changedLED_Power[index] = 1;
		phid->nextLED_Power[index] = power;
		CThread_reset_event(&phid->phid.writtenEvent);
		CThread_mutex_unlock(&phid->phid.outputLock);
		CThread_set_event(&phid->phid.writeAvailableEvent);
	}
	return EPHIDGET_OK;
}

static int CCONV
CPhidgetLED_makePacket(CPhidgetHandle phid, unsigned char *buffer,
    unsigned int *len)
{
	CPhidgetLEDHandle led = (CPhidgetLEDHandle)phid;
	
	int i;
	int numLeds = 0;


	if (!len || *len < 8)
		return EPHIDGET_INVALIDARG;
	if (!led)
		return EPHIDGET_INVALIDARG;

	if (!CPhidget_isAttached(&led->phid))
		return EPHIDGET_UNEXPECTED;


	CThread_mutex_lock(&led->phid.outputLock);
	for (i = 0; i < led->phid.attr.led.numLEDs; i++)
	{
		if (led->changedLED_Power[i] && numLeds < 4) {
			led->LED_Power[i] = led->nextLED_Power[i];
			led->changedLED_Power[i] = 0;
			led->nextLED_Power[i] = -1;
			buffer[numLeds*2] = i;
			buffer[numLeds*2+1] = led->LED_Power[i];
			numLeds++;
			led->changeRequests--;
		}
	}

	for(;numLeds<4;numLeds++)
	{
		buffer[numLeds*2] = buffer[(numLeds-1)*2];
		buffer[numLeds*2+1] = buffer[(numLeds-1)*2+1];
	}
	
	if(led->changeRequests)
		CThread_set_event(&led->phid.writeAvailableEvent);
	CThread_mutex_unlock(&led->phid.outputLock);

	return EPHIDGET_OK;
}

CGET(LED,NumLEDs,int)
	TESTPTRS(phid,pVal) 
	MASGN1(phid.attr.led.numLEDs)
}
CGETINDEX(LED,DiscreteLED,int)
	int power;
	TESTPTRS(phid,pVal) 
	TESTRNG1(phid.attr.led.numLEDs)
	power = (phid->LED_Power[Index]*100)/64;
	*pVal = power;
	return EPHIDGET_OK;
}

CSETINDEX(LED,DiscreteLED,int)
	int i, ret;
	TESTPTR(phid) 
	TESTRNG1(phid.attr.led.numLEDs)
	if (newVal > 100) return EPHIDGET_INVALIDARG;
	if(phid->phid.networkInfo)
		ADDNETWORKKEYINDEXED(Brightness, "%d")
	else
	{
		CThread_mutex_lock(&phid->phid.writelock);

		if ((phid->phid.deviceVersion >= 100) && (phid->phid.deviceVersion < 300))
		{
			i = (unsigned char)((((int)newVal) * 6400) / 10000); 
			if (i == 64) i = 63;

			ret = _sendpacket(phid, Index, i);
			CThread_mutex_unlock(&phid->phid.writelock);
			return ret;
		}

		else {
			CThread_mutex_unlock(&phid->phid.writelock);
			return EPHIDGET_UNEXPECTED;
		}
	}
	return EPHIDGET_OK;
}

CPHIDGETDATA(LED)
	phid = 0;
	return EPHIDGET_OK;
}

CPHIDGETINIT(LED)
	int i;
	for(i=0;i<phid->phid.attr.led.numLEDs;i++)
	{
		phid->changedLED_Power[i] = 0;
		phid->LED_Power[i] = 0;
		phid->nextLED_Power[i] = 0;
	}
	phid->changeRequests=0;
	return EPHIDGET_OK;
}

CCREATE(LED, PHIDDEF_LED)

