#include <string.h>
#include <assert.h>
#include "stdafx.h"
#include "cphidgetmotorcontrol.h"
#include "cusb.h"
#include "csocket.h"
#include "cthread.h"

CSENDPACKET_BUF(MotorControl, outputPacket, outputPacketLen)
CMAKEPACKET_BUF(MotorControl, outputPacket, outputPacketLen)

int PMC_set_motor(CPhidgetMotorControlInfo *phid, int Index)
{
	unsigned char buffer[6];
	int i,j;
	buffer[0] = 0;

	if (!CPhidget_isAttached(&phid->phid)) return EPHIDGET_NOTATTACHED;

	if ((phid->phid.deviceVersion >= 100) && (phid->phid.deviceVersion < 200))
	{
		// -0x7300 - +0x7300
		i = (int)((phid->motorPWMDesired[Index] * 230.0 * 128) / 100.0);
		j = (int)((phid->motorPWMDesired[Index] * 127) / 100.0);
		/* We need to implement a proper rounding mode here */
		if (i > 0x7300) i = 0x7300;
		if (i < -0x7300) i = -0x7300;
		if (j > 127) j = 127;
		if (j < -127) j = -127;
		buffer[0] = (unsigned char)Index;
		buffer[1] = (unsigned char)j;
		buffer[2] = (unsigned char)((((int)phid->motorAcceleration[Index] * 10) >> 8) & 0x03);
		buffer[3] = (unsigned char)((((int)phid->motorAcceleration[Index] * 10)) & 0xff);
		buffer[4] = (unsigned char)((i >> 8) & 0xff);
		buffer[5] = (unsigned char)(i & 0xff);
		return _sendpacket(phid, buffer, 6);
	}
	else return EPHIDGET_UNEXPECTED;
}

CFHANDLE(MotorControl,Input) int, int), void *userPtr)
ASGNFPTR(MotorControl,Input)

CFHANDLE(MotorControl,Motor) int, double ), void *userPtr)
ASGNFPTR(MotorControl,Motor)

CFHANDLE(MotorControl,Current) int, double ), void *userPtr)
ASGNFPTR(MotorControl,Current)

CPHIDGETDATA(MotorControl)
	char error_buffer[50];
	int i,j;
	
	TESTPTR(phid)


	/* Original Motor controller */
	if(phid->phid.deviceIDSpec == PHIDSPEC_MOTORCONTROL) {
		for (i = 0, j=1; i<phid->phid.attr.motorcontrol.numInputs; i++, j=j*2)
		{
			if (!(phid->inputStates[i]))
			{
				if ((buffer[0] & j))
				{
					phid->inputStates[i] = PTRUE;
					FIRE(Input) i, PTRUE);
				}
			}
			else if ((buffer[0] & j) == 0)
			{
				phid->inputStates[i] = PFALSE;
				FIRE(Input) i, PFALSE);
			}
		}
		for (i = 0; i<phid->phid.attr.motorcontrol.numMotors; i++)
		{
			j = (char)buffer[4 + i];
			j = (j * 100) / 127;
			if (phid->motorPWMCurrent[i] != j)
			{
				phid->motorPWMCurrent[i] = j;
				FIRE(Motor) i, j);
			}
		}
		
		for (j = 1, i = 0; i<phid->phid.attr.motorcontrol.numMotors; i++,j=j<<1)
		{
			if (buffer[1] & j)
			{
				sprintf(error_buffer,"Motor %d exceeded 1.5 Amp current limit", i);
				if (phid->phid.fptrError)
					phid->phid.fptrError((CPhidgetHandle)(&phid->phid), phid->phid.fptrErrorptr, EPHIDGET_OUTOFBOUNDS, error_buffer);
			}		
		}
	}

	/* HC motor controller - packets are indexed */
	else if(phid->phid.deviceIDSpec == PHIDSPEC_MOTORCONTROL_HC){
		int index = buffer[3];
		int value = 10;
		double realval = 0.0;
		
		//Only give the user the top 7-bit (signed) value
		//value = (((signed char)buffer[4]) << 8) + buffer[5];
		value = (((signed char)buffer[4]) << 8);

		//printf("LIB speed: %02x%02x\n",(unsigned char)*(&j+1), (unsigned char)j);

		//realval = (value * 100.0) / 29440.0;
		realval = (value * 100.0) / 32767.5;
		//printf("LIB: %d speed: %f %f\n",index,realval,phid->motorPWMCurrent[index]);
		if (phid->motorPWMCurrent[index] != realval)
		{
			phid->motorPWMCurrent[index] = realval;
			FIRE(Motor) index, realval);
		}
		/* current */
		realval = (unsigned int)(((unsigned char)buffer[6] << 8) | (unsigned char)buffer[7]);
		//realval = (((value * 5.0) / 524288.0) / 0.15);
		//printf("LIB: %d current: %d %f\n",index,value,realval);
		if (phid->motorSensedCurrent[index] != realval)
		{
			phid->motorSensedCurrent[index] = realval;
			FIRE(Current) index, realval);
		}
		j=0x10;
		for (i = 0; i<phid->phid.attr.motorcontrol.numMotors; i++)
		{
			if (!(buffer[1] & j))
			{
				sprintf(error_buffer,"Motor %d Error", i);
				if (phid->phid.fptrError)
					phid->phid.fptrError((CPhidgetHandle)(&phid->phid), phid->phid.fptrErrorptr, EPHIDGET_OUTOFBOUNDS, error_buffer);
			}
			j<<=1;
			if (!(buffer[1] & j))
			{
				sprintf(error_buffer,"Motor %d Error", i);
				if (phid->phid.fptrError)
					phid->phid.fptrError((CPhidgetHandle)(&phid->phid), phid->phid.fptrErrorptr, EPHIDGET_OUTOFBOUNDS, error_buffer);
			}
			j<<=1;
		}
	}
	return EPHIDGET_OK;
}

CGETINDEX(MotorControl,InputState,int)
	TESTPTRS(phid,pVal) 
	TESTRNG1(phid.attr.motorcontrol.numInputs)
	MASGN1(inputStates[Index])
}

CGET(MotorControl,NumMotors,int)
	TESTPTRS(phid,pVal) 
	MASGN1(phid.attr.motorcontrol.numMotors)
}

CGET(MotorControl,NumInputs,int)
	TESTPTRS(phid,pVal) 
	MASGN1(phid.attr.motorcontrol.numInputs)
}

CGETINDEX(MotorControl,Acceleration,double)
	TESTPTRS(phid,pVal) 
	TESTRNG1(phid.attr.motorcontrol.numMotors)
	MASGN1(motorAcceleration[Index])
}

CSETINDEX(MotorControl,Acceleration,double)
	int ret;
	TESTPTR(phid) 
	TESTRNG1(phid.attr.motorcontrol.numMotors)
	if ((newVal > 100) || (newVal < 0)) return EPHIDGET_INVALIDARG;	
	if(phid->phid.networkInfo)
		ADDNETWORKKEYINDEXED(Acceleration, "%lf")
	else
	{
		CThread_mutex_lock(&phid->phid.writelock);
		phid->motorAcceleration[Index] = newVal;
		ret = PMC_set_motor(phid, Index);
		CThread_mutex_unlock(&phid->phid.writelock);
	}
	return ret;
}

CGETINDEX(MotorControl,MotorSpeed,double)
	TESTPTRS(phid,pVal) 
	TESTRNG1(phid.attr.motorcontrol.numMotors)
	MASGN(motorPWMCurrent[Index]);
}

CSETINDEX(MotorControl,MotorSpeed,double)
	int ret;
	TESTPTR(phid) 
	TESTRNG1(phid.attr.motorcontrol.numMotors)
	if ((newVal > 100) || (newVal < -100)) return EPHIDGET_INVALIDARG;
	if(phid->phid.networkInfo)
		ADDNETWORKKEYINDEXED(Velocity, "%lf")
	else
	{
		CThread_mutex_lock(&phid->phid.writelock);	
		phid->motorPWMDesired[Index] = newVal;
		ret = PMC_set_motor(phid, Index);
		CThread_mutex_unlock(&phid->phid.writelock);
	}
	return ret;
}

CPHIDGETINIT(MotorControl)
	int i;
	for (i = 0; i<phid->phid.attr.motorcontrol.numMotors; i++)
		phid->motorAcceleration[i] = 10;
	return EPHIDGET_OK;
}

CCREATE(MotorControl, PHIDDEF_MOTORCONTROL)
