#include "stdafx.h"
#include "cusb.h"

#include <Wtypes.h>
#include <setupapi.h>
#include <wchar.h>
#include <dbt.h>
#include <Winuser.h>
#include "CPhidgetManager.h"

#ifdef _MSC_EXTENSIONS
#include "hidsdi.h"
#include "hidpi.h"
#else
#include <ddk/hidsdi.h>
#include <ddk/hidpi.h>
#endif

#include <assert.h>

HINSTANCE inst;
HWND hWnd;

/* CHID is meant to be platform dependent */

int
CUSBCloseHandle(CPhidgetHandle phid)
{
	if (!phid)
		return EPHIDGET_INVALIDARG;

	CPhidget_setStatus(phid, PHIDGET_NOTATTACHED);
	if (phid->deviceHandle == INVALID_HANDLE_VALUE)
		return EPHIDGET_NOTATTACHED;

	SetEvent((void *)phid->closeReadEvent);
	SetEvent((void *)phid->closeWriteEvent);

	CloseHandle((void *)phid->deviceHandle);

	ResetEvent((void *)phid->closeReadEvent);
	ResetEvent((void *)phid->closeWriteEvent);

	CloseHandle((void *)phid->closeReadEvent);
	CloseHandle((void *)phid->closeWriteEvent);

	phid->deviceHandle = phid->deviceHandle = INVALID_HANDLE_VALUE;

	/*DPRINT("Shut everything down\n");*/
	return EPHIDGET_OK;
}


int
CUSBSetLabel(CPhidgetHandle phid, char *buffer)
{
	return EPHIDGET_OK;
}

/* XXX This function needs error checking */
int
CUSBGetDeviceCapabilities(CPhidgetHandle phid, HANDLE dhandle)
{
	int i;
	unsigned short SerialNumber[20];
	char labelData[20];
	int serNum;
	PHIDP_PREPARSED_DATA PreparsedData;
	HIDP_CAPS Capabilities;
	HANDLE DeviceHandle = dhandle;

	HidD_GetPreparsedData (DeviceHandle, &PreparsedData);
	
	HidP_GetCaps (PreparsedData, &Capabilities);

	phid->inputReportByteLength = Capabilities.InputReportByteLength;
	phid->outputReportByteLength = Capabilities.OutputReportByteLength;
	
	HidD_FreePreparsedData(PreparsedData);

	if(HidD_GetIndexedString(DeviceHandle, 4, labelData, 20))
	{
		for(i=0;i<10;i++)
			phid->label[i] = labelData[i*2];
		phid->label[10] = '\0';
	}
	else ZEROMEM(phid->label, 10);

	if (!HidD_GetSerialNumberString(DeviceHandle, SerialNumber, 20)) return EPHIDGET_UNEXPECTED;

	swscanf((wchar_t *)(SerialNumber), L"%d", &serNum);

	if (phid->specificDevice != FALSE)
		if (phid->serialNumber != serNum)
			return EPHIDGET_NOTFOUND;
	phid->serialNumber = serNum;

	return EPHIDGET_OK;
}

//This will be simplified to just build a list and nothing else...
//it doesn't empty the list, just adds to it
int
CUSBBuildList(CPhidgetList **curList)
{
	GUID HidGuid;
	SP_DEVICE_INTERFACE_DATA devInfoData;
	HIDD_ATTRIBUTES Attributes;
	HANDLE DeviceHandle;
	HANDLE hDevInfo;
/* COMMTIMEOUTS timeOuts;*/
	int MemberIndex, i, found, interface_num;
	long Result;
	unsigned long Length;
	unsigned long Required;
	PSP_DEVICE_INTERFACE_DETAIL_DATA_W detailData;
	CPhidgetList *traverse;
	CPhidgetHandle phid;
	WCHAR * interface_spot = 0;
	Length = 0;
	MemberIndex = 0;

	HidD_GetHidGuid(&HidGuid); 
	hDevInfo = SetupDiGetClassDevs(&HidGuid, NULL, NULL, DIGCF_PRESENT |
	    DIGCF_DEVICEINTERFACE); 
	devInfoData.cbSize = sizeof(devInfoData);

	while (SetupDiEnumDeviceInterfaces(hDevInfo, 0, &HidGuid, MemberIndex,
	    &devInfoData) != 0)
	{
		Result = SetupDiGetDeviceInterfaceDetailW(hDevInfo,
		    &devInfoData, NULL, 0, &Length, NULL);
		detailData = (PSP_DEVICE_INTERFACE_DETAIL_DATA_W)malloc(Length);
		detailData->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA_W);
		Result = SetupDiGetDeviceInterfaceDetailW(hDevInfo,
		    &devInfoData, detailData, Length, &Required, NULL);
		assert(Result);
		found = 0;
		if (AttachedDevices) {
			for (traverse = AttachedDevices; traverse;
			    traverse=traverse->next) {
				if (!wcsncmp(
				    (WCHAR *)traverse->phid->CPhidgetFHandle,
				    detailData->DevicePath, 255)) {
					CList_addToList((CListHandle *)curList, traverse->phid, CPhidget_areEqual);

					found = 1;
				}
			}
		}
		if (!found) {
			interface_spot = wcsstr(detailData->DevicePath,L"mi_");
			if (interface_spot != 0)
				interface_num = (interface_spot)[4] - 48;
			else
				interface_num = 0;
			
			Attributes.Size = sizeof(Attributes);
			DeviceHandle = CreateFileW(detailData->DevicePath,
			    GENERIC_READ|GENERIC_WRITE, FILE_SHARE_READ|
			    FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 0, NULL);
			if (DeviceHandle != INVALID_HANDLE_VALUE &&
			    HidD_GetAttributes (DeviceHandle, &Attributes)) {
				const CPhidgetDeviceDef *pdd = Phid_Device_Def;
				i = 0;

				while (pdd->pdd_did) {
					if (Attributes.VendorID == pdd->pdd_vid
					    && Attributes.ProductID == pdd->pdd_pid &&
						interface_num == pdd->pdd_iid)
					{
						if((Result = CPhidget_create(&phid))) return Result;

						DPRINT("new device found\n");
						if (Attributes.VersionNumber < 0x100) 
							phid->deviceVersion = Attributes.VersionNumber *
							    100;
						else 
							phid->deviceVersion = ((Attributes.VersionNumber >>
							    8) * 100) + ((Attributes.VersionNumber &
								0xff));
						phid->deviceType = Phid_DeviceName[
						    pdd->pdd_did];

						CPhidget_setStatus(phid, PHIDGET_ATTACHED);
						phid->deviceIDSpec = i;
						phid->deviceID = pdd->pdd_did;
						phid->ProductID = Attributes.ProductID;
						phid->VendorID = Attributes.VendorID;
						
						phid->CPhidgetFHandle = malloc(wcslen(detailData->DevicePath)*sizeof(WCHAR)+10);
						wcsncpy((WCHAR *)phid->CPhidgetFHandle, detailData->DevicePath, wcslen(detailData->DevicePath)+1);
						
						phid->serialNumber = -1;
						Result=-1;
						if(!CUSBGetDeviceCapabilities(phid, DeviceHandle))
						{
							phid->specificDevice = TRUE;
							Result = CList_addToList((CListHandle *)curList, phid, CPhidget_areEqual);
						}
						if (Result)
							WARN("couldn't get new device capabilities or add to curList");
					}
					pdd++;
					i++;
				}
			}
			if (DeviceHandle != INVALID_HANDLE_VALUE)
				CloseHandle(DeviceHandle);

			if (detailData) {
				free(detailData);
				detailData = 0;
			}
		}
		if (detailData) {
			free(detailData);
			detailData = 0;
		}
		MemberIndex++;
	}
	SetupDiDestroyDeviceInfoList(hDevInfo);

	return EPHIDGET_OK;
}

void
CUSBCleanup()
{
}

/* 
CHIDOpenHandle takes a CPhidgetInfo structure, with 
ProductID/VendorID/SerialNumber filled in.

If SerialNumber is = -1, CHIDOpenHandle will return with the first available
HID Phidget that matches the ProductID/VendorID.

CHIDOpenHandle should reserve the file handles - in other words,
a future call to CHIDOpenHandle should fail.
*/

int
CUSBOpenHandle(CPhidgetHandle phid)
{
	GUID HidGuid;
	SP_DEVICE_INTERFACE_DATA devInfoData;
	HIDD_ATTRIBUTES Attributes;
	HANDLE DeviceHandle;
	HANDLE hDevInfo;
/* COMMTIMEOUTS timeOuts;*/
	int MemberIndex, i, interface_num;
	int deviceFound;
	long Result;
	unsigned long Length, Required;
	WCHAR * interface_spot = 0;
	PSP_DEVICE_INTERFACE_DETAIL_DATA_W detailData;
	unsigned char buffer[8];

	Length = 0;
	MemberIndex = deviceFound = 0;
	HidD_GetHidGuid(&HidGuid); 
	hDevInfo=SetupDiGetClassDevs (&HidGuid, NULL, NULL, DIGCF_PRESENT|DIGCF_INTERFACEDEVICE); 
	devInfoData.cbSize = sizeof(devInfoData);
	
// if (excl) exclusive = 0;
// else exclusive = FILE_SHARE_READ|FILE_SHARE_WRITE;
	while (!deviceFound && SetupDiEnumDeviceInterfaces (hDevInfo, 0,
	    &HidGuid, MemberIndex, &devInfoData) != 0) {
		Result = SetupDiGetDeviceInterfaceDetailW(hDevInfo,
		    &devInfoData, NULL, 0, &Length, NULL);
		detailData = (PSP_DEVICE_INTERFACE_DETAIL_DATA_W)malloc(Length);
		detailData->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);
		Result = SetupDiGetDeviceInterfaceDetailW(hDevInfo,
		    &devInfoData, detailData, Length, &Required, NULL);

		interface_spot = wcsstr(detailData->DevicePath,L"mi_");
		if (interface_spot != 0)
			interface_num = (interface_spot)[4] - 48;
		else
			interface_num = 0;

		DeviceHandle = CreateFileW(detailData->DevicePath,
		    GENERIC_READ|GENERIC_WRITE, 
		    FILE_SHARE_READ|FILE_SHARE_WRITE,
		    NULL, OPEN_EXISTING, 0, NULL);
		if (DeviceHandle != INVALID_HANDLE_VALUE)
		{
			Attributes.Size = sizeof(Attributes);

			if (HidD_GetAttributes (DeviceHandle, &Attributes))
			{
				const CPhidgetDeviceDef *pdd = Phid_Device_Def;
				i = 0;

				while (pdd->pdd_did) {
					if (pdd->pdd_did == phid->deviceID && Attributes.VendorID == pdd->pdd_vid && Attributes.ProductID == pdd->pdd_pid && interface_num == pdd->pdd_iid) {
						DPRINT("MATCHED");
						if (Attributes.VersionNumber < 0x100)
							phid->deviceVersion = Attributes.VersionNumber * 100;
						else
							phid->deviceVersion = ((Attributes.VersionNumber >> 8) * 100) + ((Attributes.VersionNumber & 0xff)); 
						if (!CUSBGetDeviceCapabilities(phid, DeviceHandle))
						{
							CloseHandle(DeviceHandle);
							DeviceHandle = INVALID_HANDLE_VALUE;

							phid->deviceHandle = CreateFileW(detailData->DevicePath,
							    GENERIC_READ|GENERIC_WRITE, FILE_SHARE_READ|FILE_SHARE_WRITE, NULL,
											OPEN_EXISTING, FILE_FLAG_OVERLAPPED, NULL);

							phid->closeReadEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
							phid->closeWriteEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
							if ((!phid->closeReadEvent) || (!phid->closeWriteEvent))
							{ 
								DPRINT("CreateEvent failure regarding %ls", detailData->DevicePath);
								CloseHandle(phid->deviceHandle);
								break;
							}

							//ResetEvent((void *)phid->closeReadEvent); 
							//ResetEvent((void *)phid->closeWriteEvent); 
						
							if (phid->deviceHandle != INVALID_HANDLE_VALUE)
							{
								DPRINT("deviceFound");
								deviceFound = 1;
								//CPhidget_setStatus(phid, PHIDGET_ATTACHED);
								phid->deviceIDSpec = i;
								phid->deviceType = Phid_DeviceName[
									pdd->pdd_did];

								phid->ProductID = Attributes.ProductID;
								phid->VendorID = Attributes.VendorID;
								//ZeroMemory(phid->fptrDetach,phid->totalStructLength - (int)(((int)(phid->fptrDetach)) - ((int)phid->deviceID)));

								phid->attr = pdd->pdd_attr;

								if ((Phid_Device_Def_WorkAroundA[i][0]) &&
									(Phid_Device_Def_WorkAroundA[i][1] >= phid->deviceVersion))
								{
									DPRINT("Sending workaround startup packet");
									ZEROMEM(buffer,8);
									CUSBSendPacket((CPhidgetHandle)phid, buffer);
								}

								//free(detailData);
								break;
							} else
								DPRINT("could not reopen %ls", detailData->DevicePath);
						}
					} else {
						DPRINT("no match with %x,%x",
						    pdd->pdd_vid, pdd->pdd_pid);
					}
					pdd++;
					i++;
				}
			} else
				DPRINT("could not get HID attributes for %ls",
				    detailData->DevicePath);
			if (DeviceHandle != INVALID_HANDLE_VALUE)
				CloseHandle(DeviceHandle);
		} else
			DPRINT("could not open %ls", detailData->DevicePath);

		if(detailData)
		{
			free(detailData);
			detailData = 0;
		}

		MemberIndex++;
		if (deviceFound)
			break;
	}
	SetupDiDestroyDeviceInfoList(hDevInfo);
	if (deviceFound)
		return EPHIDGET_OK;
	else
		return EPHIDGET_NOTFOUND;
}

/* CHIDSendPacket compensates for an oddity in the Windows HID driver.
Windows HID expects the first byte of the packet to be a zero byte,
thus offsetting all the bytes by 1.  None of the other operating systems
have this behaviour, as far as I know.  Rather than offsetting all bytes 
in the functions that format the packets, I deal with it here, for maximum
code reuse on Linux/MAC. */

int CCONV
CUSBSendPacket(CPhidgetHandle phid, unsigned char *buffer)
{
	unsigned char outbuffer[100];
	int i;
	OVERLAPPED writeEventStruct;
	unsigned long BytesWritten = 0;

	outbuffer[0] = 0;
	for (i = 0; i<phid->outputReportByteLength; i++)
		outbuffer[i+1] = buffer[i];

	if (!CPhidget_isAttached(phid))
		return EPHIDGET_NOTATTACHED;	

	if (phid->deviceHandle == INVALID_HANDLE_VALUE)
	{
		WARN("Handle for writing is not valid");
		return EPHIDGET_UNEXPECTED;
	}

	writeEventStruct.Offset = 0;
	writeEventStruct.OffsetHigh = 0;
	writeEventStruct.hEvent = (void *)phid->closeWriteEvent;


	if (!WriteFile((void *)phid->deviceHandle, outbuffer, phid->outputReportByteLength, &BytesWritten, &writeEventStruct))
	{
		switch(GetLastError())
		{
		case ERROR_IO_PENDING:
			WaitForSingleObject ((void *)phid->closeWriteEvent, (DWORD)1000);             
			//DPRINT("%dR", WaitForSingleObject ((void *)phid->closeWriteEvent, (DWORD)1000));     
			// 258 Indicates WAIT_TIMEOUT
			break;
		case ERROR_IO_INCOMPLETE:
			return EPHIDGET_INTERRUPTED;
		case ERROR_DEVICE_NOT_CONNECTED:
			//DPRINT("Device Not Connected\n");
			return CPhidgetDetachEvent(phid);
		default:
			WARN("Unknown error returned from Writefile - %d\n",
			    (int)GetLastError());
			return EPHIDGET_UNEXPECTED;
		}
	}

	if (phid->outputReportByteLength != BytesWritten &&
	    !GetOverlappedResult((void *)phid->deviceHandle,
	    &writeEventStruct, &BytesWritten, FALSE))
	{
		WARN("Failure 1 in CHIDSendPacket Report Length"
		    "- %d bytes written - %d\n",
		    (int)phid->outputReportByteLength, (int)BytesWritten);
		return EPHIDGET_UNEXPECTED;
	}
 
	if (BytesWritten != phid->outputReportByteLength) {
		WARN("Failure 2 in CHIDSendPacket Report Length"
		    "- %d Bytes Written - %d\n", (int)phid->outputReportByteLength,
		    (int)BytesWritten);
		return EPHIDGET_UNEXPECTED;
	}
	return EPHIDGET_OK;
}


/* CHIDReadPacket compensates for an oddity in the Windows HID driver.
Windows HID returns an extra byte on the beginning of the packet, always
set to zero.  None of the other operating systems have this behaviour, 
as far as I know.  Rather than offsetting all bytes in the functions that
parse the packets, I deal with it here, for maximum code reuse on Linux/MAC. */

/* Buffer should be at least 8 bytes long */
int CCONV
CUSBReadPacket(CPhidgetHandle phid, unsigned char *buffer)
{
	unsigned char inbuf[9];
	int i;
	OVERLAPPED readEventStruct;

	unsigned long BytesRead = 0;

	if (phid->deviceHandle == (HANDLE)INVALID_HANDLE_VALUE)
		return EPHIDGET_UNEXPECTED;

	if (CPhidget_isAttached(phid))
	{
		readEventStruct.Offset = 0;
		readEventStruct.OffsetHigh = 0;
		readEventStruct.hEvent = (void *)phid->closeReadEvent;
		//printf("CUSBWindows Read thread running\n");
		if (!ReadFile((void *)phid->deviceHandle, inbuf, phid->inputReportByteLength,&BytesRead,&readEventStruct))
		{
			switch(GetLastError())
			{
			case ERROR_IO_PENDING:
				WaitForSingleObject ((void *)phid->closeReadEvent, (DWORD)INFINITE);             
				break;
			case ERROR_DEVICE_NOT_CONNECTED:
				CPhidgetDetachEvent(phid);
				return EPHIDGET_NOTATTACHED;
			case ERROR_IO_INCOMPLETE:
				/*CHIDCloseHandle(phid);*/
				return EPHIDGET_INTERRUPTED;
			case ERROR_INVALID_USER_BUFFER: //this happens when we try to read a device with size 0 read buffer
				return EPHIDGET_UNEXPECTED;
			default:
				WARN("Unknown Error returned from read - %d\n",
				    (int)GetLastError());
				return EPHIDGET_UNEXPECTED;
			}
			
		}

		if ((phid->inputReportByteLength != BytesRead) && (!GetOverlappedResult((void *)phid->deviceHandle, &readEventStruct, &BytesRead, FALSE)))
		{
			return EPHIDGET_UNEXPECTED;
		}
 
		if (phid->inputReportByteLength != BytesRead) 
		{
			//DPRINT("Error - %d\n", GetLastError());
			return EPHIDGET_UNEXPECTED;
		}

		for (i = 0; i<8; i++)
			buffer[i] = inbuf[i + 1];
		return EPHIDGET_OK;
	}
	else return EPHIDGET_NOTATTACHED;
}
