#include "includes/osmolation.hpp"

void Osmolation::init()
{
  state=0;
  state_sauv=0;
  captor_high=0;
  stop=false;
  reserv_mail=false;
  empty_reserv=0;
  ini=0;
  full=false;
  flow=0; // used to count the nb of impulse from flowmeter
  flowinit=0; // used to count the nb of impulse from flowmeter
  stopper_osmo=false;	


}

void Osmolation::forceState(short forcedState,K8055 &k,config &cfg)
{
  if (state!=forcedState)
    {
      state_sauv=state;
      state=forcedState;
      k.change_state(cfg.OUTOSMOLATION,cfg.OUTOSMOLATION_CARD,state,"osmolation forcée");
    }
}

//inserting water quantity measured with flowmeter in bd (osmolation)
void Osmolation::quantity_flow(connexion &c, K8055 &k)
{

flowinit=(double)k.readcounter(cfg.FLOW,cfg.FLOW_CARD);
c.insert_osmolation((flowinit+flow)/cfg.FLOWIMPULSE);
if (DEBUG==1) cout << "flow quant:" << (flowinit+flow)/cfg.FLOWIMPULSE << "imp: "<< flowinit << endl;
}


//inserting water quantity estimated in bd (osmolation)
void Osmolation::quantity_est(connexion &c, double dif)
{
  c.insert_osmolation((dif/3600)*cfg.DEBIT);
}

void Osmolation::changeState (short state,K8055 &k,config &cfg)
{
  this->state=state;
  state_sauv=state;
  k.change_state(cfg.OUTOSMOLATION,cfg.OUTOSMOLATION_CARD,state,"osmolation");
}

void Osmolation::stopper (K8055 & k,config& cfg,string s)
{
  stop=false;
  state=0;
  state_sauv=state;
  k.change_state(cfg.OUTOSMOLATION,cfg.OUTOSMOLATION_CARD,state,s);
}

void Osmolation::changing(connexion &c,K8055 &k,config &cfg)
{
  double dif=0;
  double flowend=0; // used to count the nb of impulse from flowmeter

//state of osmolations 
  if(cfg.forced_osmolation!=0)
    forceState(cfg.forced_osmolation-1,k,cfg);
  else
    {
      if (state!=state_sauv)
    	changeState(state_sauv,k,cfg);
      
	if (stopper_osmo==false && empty_reserv==0 && ini==1 && stop==false)
	{
//osmolation pump 
	  time (&timer);
	  stop=true;
	  reserv_mail=false;
	}
      
      if(stop==true)
	{
	  time ( &rawtime );
	  dif = difftime (rawtime,timer);
	  if( (dif >= cfg.TOSMOLATION) && (ini==0)) 
	    {
		// stopping osmolation	      
	      stop=false;
	      changeState(0,k,cfg);
		//inserting water quantity in bd water
		if(cfg.FLOW!=0) 
              	quantity_flow(c,k);  
			      
	      		else quantity_est(c,dif);
	      
	    }  
	  else
	  {	
	// state changing init osmolation if not running	    
	    if(state==0)
		{
		if (cfg.FLOW !=0) // reset the impulse counter connected on the flowmeter
                	{
		  	flowend=(double)k.readcounter(cfg.FLOW,cfg.FLOW_CARD);
			flow=flowend-flowinit; // check if some impulses have been forgotten 
                	k.resetcounter(cfg.FLOW,cfg.FLOW_CARD);
                	}  
		changeState(1,k,cfg);
		}
	}
	}	
	if (stopper_osmo==true)
		{
		time ( &rawtime );
	  	dif = difftime (rawtime,reserv_stop);
	  	if(dif >= 10)
			{
			 stopper_osmo=false; 
			 // no waterin the reserv => sending an email alert
	  		  send_mail("plus d'eau !!!",c);	
	  		 if(state==1)
	    			{
	      			stopper (k,cfg,"plus d'eau osmolation");
				if(cfg.FLOW!=0) 
              			quantity_flow(c,k);  
			      
	      			else quantity_est(c,dif);
	    			}
	  		}

	
 
		}	
	else 
		{
			if (empty_reserv==1 && reserv_mail==false)
			{
			time(&reserv_stop);
			stopper_osmo=true;
			reserv_mail=true;
			}		      
		}

      
      if (captor_high==1 && full==false)
	{
	  
	  if(state==1)
	    {
	      // decantation is full, osmolation captor is not working => sending an email alert
	      send_mail("decantation full of water, pb with osmolation captor !!!",c);	
	      full=true;
	      stopper(k,cfg,"decantation pleine, osmolation");
      		if(cfg.FLOW!=0) 
              	quantity_flow(c,k);  
			      
      		else quantity_est(c,dif);

	    }
	  
	}
      if (captor_high==0 && full==true)
	full=false;
    }
}
